%token ADD_ MIN_ MUL_ DIV_ SEMI_ INT_ FLOAT_ DOT_ COMMA_ LEFT_ROUND_
%token RIGHT_ROUND_ IF_ THEN_ ELSE_ ELSIF_ BEGIN_ END_ WHILE_ FOR_
%token REPEAT_ DO_ MODULE_ ID_ ASSIGN_ EQUAL_ NOTEQUAL_ LESS_ GREAT_ 
%token LESSEQUAL_ GREATEQUAL_ OR_ AND_	TILDE_ TO_ CONST_ VAR_ COLON_
%token BOOLEANTYPE_ INTEGERTYPE_ REALTYPE_ UNTIL_

%union
{
  int i;
  char ch;
}
%%

module:						MODULE_ ID_ SEMI_ declseq 
								BEGIN_ statseq END_ ID_ DOT_				 	{ debug("module");
																						}
								;

declseq:						/* empty */											{ debug("empty declseq"); }
						|		consthead declseq									{ debug("consthead declseq"); }
						|		varhead declseq									{ debug("varhead declseq"); }
								;

consthead:					CONST_ constantdeclaration		  				{ debug("const head"); }
								;

constantdeclaration:		constantdeclaration 
									ID_ EQUAL_ expression SEMI_				{ debug("constant declaration"); }
						|		ID_ EQUAL_ expression SEMI_					{ debug("constant declaration"); }
								;

varhead:						VAR_ variabledeclaration						{ debug("var head"); }
								;

variabledeclaration:		variabledeclaration 
									identlist COLON_ type SEMI_				{ debug("variable declaration"); }
						|		identlist COLON_ type SEMI_					{ debug("variable declaration"); }
								;

identlist:					ID_													{ debug("1 indentlist"); }
						|		identlist COMMA_ ID_								{ debug("2 indentlist"); }
								;

type:							BOOLEANTYPE_										{ debug("1 type"); }
						|		REALTYPE_											{ debug("2 type"); }
						|		INTEGERTYPE_										{ debug("3 type"); }
						;

statseq:						/* empty */
						|		statement											{ debug("1 statseq"); }
                  |		statseq SEMI_ statement							{ debug("1 statseq"); }
								;														

statement:					assignment											{ debug("1 statement"); }
						|		ifstatement											{ debug("2 statement"); }
						|		whilestatement										{ debug("3 statement"); }
						|		forstatement										{ debug("4 statement"); }
						|		repeatstatement									{ debug("5 statement"); }
								;

assignment:					designator ASSIGN_ expression					{ debug("assignment"); }
								;

ifstatement:				ifpart END_											{ debug("1 ifstat"); }
						|		ifpart elseifpart END_							{ debug("2 ifstat"); }
						|		ifpart elsepart END_								{ debug("3 ifstat"); }
						|		ifpart elseifpart elsepart END_				{ debug("4 ifstat"); }
								;

ifpart:						IF_ expression THEN_ statseq					{ debug("ifpart");	 }
								;

elseifpart:					ELSIF_ expression THEN_ statseq				{ debug("1 elseifpart");}
						|		elseifpart 
								ELSIF_ expression THEN_ statseq				{ debug("2 elseifpart"); }
								;

elsepart:					ELSE_	statseq										{ debug("elsepart");}
								;

whilestatement:			WHILE_ expression DO_ statseq END_			{ debug("while");}
								;

forstatement:				FOR_ ID_ ASSIGN_ expression TO_ expression 
									DO_ statseq END_								{ debug("for statement"); }
								;

repeatstatement:			REPEAT_ statseq UNTIL_ expression			{ debug("repeat"); }
								;

expression:					simpleexpression									{ debug("1 expression"); }
						|		simpleexpression relation simpleexpression{ debug("2 expression"); }
								;

relation:					EQUAL_												{ debug("1 relation"); } 
						| 		NOTEQUAL_ 											{ debug("2 relation"); } 
						| 		LESS_ 												{ debug("3 relation"); } 
						| 		GREAT_ 												{ debug("4 relation"); } 
						| 		LESSEQUAL_ 											{ debug("5 relation"); } 
						| 		GREATEQUAL_											{ debug("6 relation"); } 
								;

simpleexpression:			term 													{ debug("1 simpleexpression");}	
						|		ADD_ term											{ debug("2 simpleexpression");}
						|		MIN_ term											{ debug("3 simpleexpression");}
						|		simpleexpression addoperator term			{ debug("4 simpleexpression");}	
								;

addoperator:				ADD_													{ debug("1 addoperator");}	
						|		MIN_													{ debug("2 addoperator");}	
						|		OR_													{ debug("3 addoperator");}	
								;

muloperator:				MUL_													{ debug("1 muloperator");}
						|		DIV_													{ debug("1 muloperator");}
						|		AND_													{ debug("1 muloperator");}
								;

term:							factor												{ debug("1 term"); }
						|		term muloperator factor							{ debug("2 term"); }	
								;

factor:						number												{ debug("1 factor"); }
						|		designator											{ debug("2 factor"); }
						|		LEFT_ROUND_ expression RIGHT_ROUND_			{ debug("3 factor"); }
						|		TILDE_ factor										{ debug("4 factor"); }
								;

number:						INT_													{ debug("1 number"); }
						|		FLOAT_												{ debug("2 number"); }
								;

designator:					qualident											{ debug("1 designator"); }
								;

qualident:					ID_ DOT_ ID_										{ debug("1 qualident"); }
						|		ID_													{ debug("2 qualident"); }
								;
